/*
 * eID Chrome OS project.
 *
 * Copyright 2018 e-Contract.be BVBA. All rights reserved.
 * e-Contract.be BVBA proprietary/confidential. Use is subject to license terms.
 */

/**
 * Draws a battery on an HTML5 canvas.
 * 
 * @param {string} id the id of the HTML5 canvas element.
 * @param {number} level the battery level from 0 to 100.
 */
function drawBattery(id, level) {
    if (level > 100) {
        level = 100;
    }
    if (level < 0) {
        level = 0;
    }
    var canvas = document.querySelector("#" + id);
    if (canvas === null) {
        return;
    }
    var context = canvas.getContext("2d");
    var height = canvas.height - 5;
    var length = canvas.width - 5;
    var tip = 7;
    var offset = 2;
    context.moveTo(offset, offset);
    context.lineTo(length - tip + offset, 0 + offset);
    context.lineTo(length - tip + offset, (height - tip) / 2 + offset);
    context.lineTo(length + offset, (height - tip) / 2 + offset);
    context.lineTo(length + offset, (height + tip) / 2 + offset);
    context.lineTo(length - tip + offset, (height + tip) / 2 + offset);
    context.lineTo(length - tip + offset, height + offset);
    context.lineTo(offset, height + offset);
    context.lineTo(offset, offset);
    context.lineWidth = 2;
    context.closePath();
    context.strokeStyle = "#a0a0a0";
    context.stroke();

    if (level <= 25) {
        context.fillStyle = "#ff0000";
    } else {
        context.fillStyle = "#00ff00";
    }
    var border = 3;
    var levelX = border + (length - tip - 3 * border) * level / 100;
    context.fillRect(border + offset, border + offset, levelX, height - 2 * border);

    context.fillStyle = "#ffffff";
    context.fillRect(levelX + offset, border + offset, length - levelX - tip - border, height - 2 * border);
}

/**
 * Clears the battery drawing from the HTML5 canvas.
 * 
 * @param {string} id the id of the HTML5 canvas element.
 */
function clearBattery(id) {
    var canvas = document.querySelector("#" + id);
    if (canvas === null) {
        return;
    }
    var context = canvas.getContext("2d");
    var height = canvas.height;
    var width = canvas.width;
    context.fillStyle = "#ffffff";
    context.fillRect(0, 0, width, height);
}