/*
 * eID Android project.
 *
 * Copyright 2017 e-Contract.be BVBA. All rights reserved.
 * e-Contract.be BVBA proprietary/confidential. Use is subject to license terms.
 */


"use strict";

var offlineUI = new OfflineUI();

// array buffer to string
function ab2str(buf) {
    return String.fromCharCode.apply(null, new Uint8Array(buf));
}

function str2ab(str) {
    var buf = new ArrayBuffer(str.length);
    var bufView = new Uint8Array(buf);
    for (var i = 0, strLen = str.length; i < strLen; i++) {
        bufView[i] = str.charCodeAt(i);
    }
    return buf;
}

function parseHttpHeaders(httpRequestArrayBuffer) {
    var httpRequestData = ab2str(httpRequestArrayBuffer);
    var headerLines = httpRequestData.split('\r\n');
    var headers = {};
    headerLines.forEach(function (headerLine) {
        console.log("header line: " + headerLine);
        console.log("header line length: " + headerLine.length);
        if (headerLine.length === 0) {
            return;
        }

        if (headerLine.indexOf(":") === -1) {
            return;
        }

        var headerName = headerLine.substring(0, headerLine.indexOf(":"));
        var headerValue = headerLine.substring(headerLine.indexOf(":") + 1).trim();
        console.log("header: " + headerName + " = " + headerValue);
        headers[headerName] = headerValue;
    });
    return headers;
}

function parseHttpBody(httpRequestArrayBuffer) {
    var httpRequestData = ab2str(httpRequestArrayBuffer);
    var httpBlocks = httpRequestData.split('\r\n\r\n');
    if (httpBlocks.length > 1) {
        return httpBlocks[1];
    }
    return "";
}

function parseHttpMethod(httpRequestArrayBuffer) {
    var httpRequestData = ab2str(httpRequestArrayBuffer);
    var httpLines = httpRequestData.split('\r\n');
    var methodLine = httpLines[0];
    console.log("HTTP method line: " + methodLine);
    var method = methodLine.substring(0, methodLine.indexOf(" "));
    console.log("method: \"" + method + "\"");
    return method;
}

function readIdentityOfflineBluetooth(options, clientSocketId, headers) {
    document.querySelector("#main").hidden = true;
    offlineUI.setStatusText("Please enable your bluetooth smart card reader...", "INFO");
    app.btbeid.start(offlineUI, function () {
        app.btbeid.cardPowerOn(function (atr) {
            var tasks = [
                app.btbeid.readIdentityFileTask("identity")
            ];
            if (options.includeAddress) {
                tasks.push(app.btbeid.readAddressFileTask("address"));
            }
            app.btbeid.runTasks(tasks, function (data) {
                var identity = parseIdentityFile(data.identity);
                var responseData = {
                    identity: identity
                };
                if (options.includeAddress) {
                    console.log("address file: " + app.btbeid.Uint8ArrayToHex(data.address));
                    var address = parseAddressFile(data.address);
                    responseData.address = address;
                }
                app.btbeid.powerOffAndClose(function () {
                    var buffer = str2ab("HTTP/1.1 200 OK\n" +
                            "Content-Type: application/vnd.api+json\n" +
                            "Access-Control-Allow-Origin: " + headers["Origin"] + "\n" +
                            "Access-Control-Allow-Headers: " + headers["Access-Control-Request-Headers"] + "\n" +
                            "Connection: closed\n\n" +
                            JSON.stringify(responseData));

                    chrome.sockets.tcp.send(clientSocketId, buffer, function (info) {
                        console.log("result code: " + info.resultCode);
                        if (info.resultCode !== 0) {
                            console.error("Send failed.");
                        }
                        chrome.sockets.tcp.close(clientSocketId);
                    });
                });
            });
        });
    }, function (errorObject) {
        offlineUI.setStatusText(errorObject.message);
        var buffer = str2ab("HTTP/1.1 500 Internal Server Error\n" +
                "Content-Type: application/vnd.api+json\n" +
                "Access-Control-Allow-Origin: " + headers["Origin"] + "\n" +
                "Access-Control-Allow-Headers: " + headers["Access-Control-Request-Headers"] + "\n" +
                "Connection: closed\n\n" +
                JSON.stringify(errorObject));

        chrome.sockets.tcp.send(clientSocketId, buffer, function (info) {
            console.log("result code: " + info.resultCode);
            if (info.resultCode !== 0) {
                console.error("Send failed.");
            }
            chrome.sockets.tcp.close(clientSocketId);
        });
    });
}

function readIdentityOffline(options, clientSocketId, headers) {
    document.querySelector("#main").hidden = true;
    selectCordovaAndroidCCIDDevice(function (devices) {
        if (devices.length === 0) {
            console.log("no device available");
            offlineUI.setStatusText("Please connect a smart card reader...", "WARNING");
            setTimeout(function () {
                // next construct avoids burning battery
                readIdentityOffline(options, clientSocketId, headers);
            }, 500);
            return;
        }
        console.log("start reading...");
        startBeID(devices, offlineUI, function () {
            var beid = this;
            var tasks = [
                beid.readIdentityFileTask("identity")
            ];
            if (options.includeAddress) {
                tasks.push(beid.readAddressFileTask("address"));
            }
            beid.runTasks(tasks, function (data) {
                var identity = parseIdentityFile(data.identity);
                var responseData = {
                    identity: identity
                };
                if (options.includeAddress) {
                    console.log("address file: " + beid.Uint8ArrayToHex(data.address));
                    var address = parseAddressFile(data.address);
                    responseData.address = address;
                }
                beid.powerOffAndClose(function () {
                    var buffer = str2ab("HTTP/1.1 200 OK\n" +
                            "Content-Type: application/vnd.api+json\n" +
                            "Access-Control-Allow-Origin: " + headers["Origin"] + "\n" +
                            "Access-Control-Allow-Headers: " + headers["Access-Control-Request-Headers"] + "\n" +
                            "Connection: closed\n\n" +
                            JSON.stringify(responseData));

                    chrome.sockets.tcp.send(clientSocketId, buffer, function (info) {
                        console.log("result code: " + info.resultCode);
                        if (info.resultCode !== 0) {
                            console.error("Send failed.");
                        }
                        chrome.sockets.tcp.close(clientSocketId);
                    });
                });
            });
        }, function (errorObject) {
            var buffer = str2ab("HTTP/1.1 500 Internal Server Error\n" +
                    "Content-Type: application/vnd.api+json\n" +
                    "Access-Control-Allow-Origin: " + headers["Origin"] + "\n" +
                    "Access-Control-Allow-Headers: " + headers["Access-Control-Request-Headers"] + "\n" +
                    "Connection: closed\n\n" +
                    JSON.stringify(errorObject));

            chrome.sockets.tcp.send(clientSocketId, buffer, function (info) {
                console.log("result code: " + info.resultCode);
                if (info.resultCode !== 0) {
                    console.error("Send failed.");
                }
                chrome.sockets.tcp.close(clientSocketId);
            });
        });
    });
}

function onReceiveCallback(receiveInfo) {
    console.log("onReceive");
    var method = parseHttpMethod(receiveInfo.data);
    var headers = parseHttpHeaders(receiveInfo.data);
    var clientSocketId = receiveInfo.socketId;

    if (method === "OPTIONS") {
        var buffer = str2ab("HTTP/1.1 200 OK\n" +
                "Content-Length: 0\n" +
                "Connection: closed\n" +
                "Access-Control-Allow-Methods: " + headers["Access-Control-Request-Method"] + "\n" +
                "Access-Control-Allow-Origin: " + headers["Origin"] + "\n" +
                "Access-Control-Allow-Headers: " + headers["Access-Control-Request-Headers"] + "\n");

        chrome.sockets.tcp.send(clientSocketId, buffer, function (info) {
            console.log("result code: " + info.resultCode);
            if (info.resultCode !== 0) {
                console.error("Send failed.");
            }
            chrome.sockets.tcp.close(clientSocketId);
        });
        return;
    }

    if (method !== "POST") {
        return;
    }
    var httpBody = parseHttpBody(receiveInfo.data);
    console.log("HTTP body: " + httpBody);
    var options = JSON.parse(httpBody);

    var readerSelection = window.localStorage.getItem("readerSelection");
    if (readerSelection === null) {
        readerSelection = DEFAULT_READER_SELECTION;
    }
    switch (readerSelection) {
        case "both":
            console.log("both options");
            break;
        case "usb":
            readIdentityOffline(options, clientSocketId, headers);
            break;
        case "bluetooth":
            console.log("bluetooth");
            readIdentityOfflineBluetooth(options, clientSocketId, headers);
            break;
    }
}

function onAcceptCallback(acceptInfo, serverSocketId) {
    console.log("onAccept");
    if (acceptInfo.socketId !== serverSocketId) {
        return;
    }
    chrome.sockets.tcp.setPaused(acceptInfo.clientSocketId, false);
}

function onListenCallback(resultCode) {
    console.log("listen callback called");
    if (resultCode < 0) {
        console.error("Error listening:" + chrome.runtime.lastError.message);
        return;
    }
}

function onAcceptError(info) {
    console.log("on accept error");
}

function listenAndAccept(socketId) {
    chrome.sockets.tcpServer.listen(socketId, "127.0.0.1", 0xbe1d, function (resultCode) {
        onListenCallback(resultCode);
    });
    chrome.sockets.tcpServer.onAccept.addListener(function (acceptInfo) {
        onAcceptCallback(acceptInfo, socketId);
    });
    chrome.sockets.tcpServer.onAcceptError.addListener(onAcceptError);
    chrome.sockets.tcp.onReceive.addListener(function (receiveInfo) {
        onReceiveCallback(receiveInfo);
    });
    chrome.sockets.tcp.onReceiveError.addListener(function (info) {
        console.error("on receive error: ", info);
    });
}

function initOffline() {
    console.log("init offline");
    chrome.sockets.tcpServer.create({}, function (createInfo) {
        console.log("tcpServer created");
        listenAndAccept(createInfo.socketId);
    });
}


function OfflineUI() {
}

OfflineUI.prototype.setStatusText = function (status, level) {
    console.log("status: " + status);
    window.plugins.toast.hide();
    if (typeof status !== "undefined") {
        window.plugins.toast.show("eID: " + status, "long", "center");
    }
};

OfflineUI.prototype.getPIN = function (applicationUrl, pinCallback) {
};

OfflineUI.prototype.showPinpad = function (applicationUrl) {
};

OfflineUI.prototype.hidePinpad = function () {
};

OfflineUI.prototype.reset = function () {
};

OfflineUI.prototype.progress = function (active) {
};

OfflineUI.prototype.setRetriesLeft = function (retries) {
};